package com.school.project.controller;

import com.school.project.dto.DepartmentDTO;
import com.school.project.service.department.DepartmentServiceImp;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.server.ResponseStatusException;

import java.util.List;
import java.util.NoSuchElementException;
import java.util.Optional;

@RestController
@RequestMapping("/department")
public class DepartmentController {
    @Autowired(required = true)
    public DepartmentServiceImp departmentServiceImp;

    @GetMapping("")
    public ResponseEntity<List<DepartmentDTO>> readAllDepartments() {
        List<DepartmentDTO> departmentDTO = departmentServiceImp.readDepartment();

        if (departmentDTO.isEmpty()) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).build();
        } else {
            return ResponseEntity.of(Optional.of(departmentDTO));
        }
    }

    @GetMapping("/findId/{id}")
    public DepartmentDTO readDepartmentById(@PathVariable("id") Long id) {
        try {
            return departmentServiceImp.readDepartmentById(id);
        } catch (NoSuchElementException ex) {
            throw new ResponseStatusException(HttpStatus.NOT_FOUND, "Department Not Found With This Id", ex);
        }
    }

    @GetMapping("/findName/{name}")
    public ResponseEntity<DepartmentDTO> findDepartmentByName(@PathVariable("name") String name) {
        DepartmentDTO departmentDTO = departmentServiceImp.findDepartmentByName(name);

        if (departmentDTO == null) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).build();
        } else {
            return ResponseEntity.of(Optional.of(departmentDTO));
        }
    }

    @PostMapping("/save")
    public ResponseEntity<DepartmentDTO> saveDepartment(@RequestBody DepartmentDTO department) {

        try {
            DepartmentDTO departmentDTO = departmentServiceImp.saveDepartment(department);

            return ResponseEntity.status(HttpStatus.OK).body(departmentDTO);

        } catch (Exception exception) {

            return ResponseEntity.status(HttpStatus.BAD_REQUEST).build();

        }

    }

    @PutMapping("/update/{id}")
    public ResponseEntity<DepartmentDTO> updateDepartmentById(@RequestBody DepartmentDTO department, @PathVariable("id") Long id) {
        try {
            DepartmentDTO updateDepartment = departmentServiceImp.updateDepartment(department, id);
            return ResponseEntity.status(HttpStatus.OK).body(updateDepartment);
        } catch (NoSuchElementException e) {
            return ResponseEntity.status(HttpStatus.UNPROCESSABLE_ENTITY).build();
        }
    }

    @DeleteMapping("/delete/{id}")
    public ResponseEntity<Object> deleteDepartmentById(@PathVariable("id") Long id) {
        try {
            departmentServiceImp.deleteDepartmentById(id);
            return ResponseEntity.status(HttpStatus.NO_CONTENT).build();
        } catch (EmptyResultDataAccessException e) {
            return ResponseEntity.status(HttpStatus.UNPROCESSABLE_ENTITY).build();
        }
    }
}
