package com.school.project.controller;

import com.school.project.dto.StudentDTO;
import com.school.project.service.student.StudentServiceImp;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.NoSuchElementException;

@RestController
@RequestMapping("/student")
public class StudentController {
    @Autowired(required = true)
    public StudentServiceImp studentServiceImp;



    @PostMapping("/save")
    public ResponseEntity<StudentDTO> saveStudent(@RequestBody StudentDTO studentDTO) {

        StudentDTO student = studentServiceImp.saveStudent(studentDTO);

        if (student == null) {
            return ResponseEntity.status(HttpStatus.UNPROCESSABLE_ENTITY).build();
        } else {
            return ResponseEntity.status(HttpStatus.OK).body(student);
        }
    }

    @GetMapping("/findAll")
    public ResponseEntity<List<StudentDTO>> readAllStudents() {
        List<StudentDTO> studentsDTO = studentServiceImp.readAllStudents();

        if (studentsDTO.size() <= 0) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).build();
        } else {
            return ResponseEntity.status(HttpStatus.OK).body(studentsDTO);
        }
    }

    @GetMapping("/find/{id}")
    public ResponseEntity<StudentDTO> readStudentById(@PathVariable("id") Long id) {

        try {
            StudentDTO dto = studentServiceImp.readStudentById(id);
            return ResponseEntity.status(HttpStatus.OK).body(dto);
        } catch (NoSuchElementException e) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).build();
        }

    }

    @PutMapping("/update/{id}")
    public ResponseEntity<StudentDTO> updateStudentById(@RequestBody StudentDTO studentDTO, @PathVariable("id") Long id) {
        try {
            StudentDTO updateStudent = studentServiceImp.updateStudentById(studentDTO, id);
            return ResponseEntity.status(HttpStatus.OK).body(updateStudent);
        } catch (NoSuchElementException e) {
            return ResponseEntity.status(HttpStatus.UNPROCESSABLE_ENTITY).build();
        }

    }

    @DeleteMapping("/delete/{id}")
    public ResponseEntity deleteStudentById(@PathVariable("id") Long id) {
        try {
            studentServiceImp.deleteStudentById(id);
            return ResponseEntity.status(HttpStatus.NO_CONTENT).build();
        } catch (EmptyResultDataAccessException e) {
            return ResponseEntity.status(HttpStatus.UNPROCESSABLE_ENTITY).build();
        }
    }
}