package com.school.project.controller;

import com.school.project.dto.TeacherDTO;
import com.school.project.service.teacher.TeacherServiceImp;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.NoSuchElementException;

@RestController
@ComponentScan(basePackages = "com.school.project.service.teacher.TeacherService")
@RequestMapping("/teacher")
public class TeacherController {

    @Autowired(required = true)
    public TeacherServiceImp teacherServiceImp;

    @PostMapping("/save")
    public ResponseEntity<TeacherDTO> saveTeacher(@RequestBody TeacherDTO teacherDTO) {
        TeacherDTO dto = teacherServiceImp.saveTeacher(teacherDTO);

        if (dto == null) {
            return ResponseEntity.status(HttpStatus.UNPROCESSABLE_ENTITY).build();
        } else {
            return ResponseEntity.status(HttpStatus.OK).body(dto);
        }
    }

    @GetMapping("/findAll")
    public ResponseEntity<List<TeacherDTO>> readAllTeachers() {
        List<TeacherDTO> teacherDTOS = teacherServiceImp.readAllTeachers();

        if (teacherDTOS.size() <= 0) {
            return ResponseEntity.status(HttpStatus.UNPROCESSABLE_ENTITY).build();
        } else {
            return ResponseEntity.status(HttpStatus.OK).body(teacherDTOS);
        }
    }

    @GetMapping("/find/{id}")
    public ResponseEntity<TeacherDTO> readTeacherById(@PathVariable("id") Long id) {

        try {
            TeacherDTO teacherDTO = teacherServiceImp.readTeacherById(id);
            return ResponseEntity.status(HttpStatus.OK).body(teacherDTO);
        } catch (NoSuchElementException e) {
            return ResponseEntity.status(HttpStatus.UNPROCESSABLE_ENTITY).build();
        }

    }

    @PutMapping("/update/{id}")
    public ResponseEntity<TeacherDTO> updateTeacherById(@RequestBody TeacherDTO teacherDTO, @PathVariable("id") Long id) {
        try {
            TeacherDTO updateTeacher = teacherServiceImp.updateTeacher(teacherDTO, id);
            return ResponseEntity.status(HttpStatus.OK).body(updateTeacher);
        } catch (NoSuchElementException e) {
            return ResponseEntity.status(HttpStatus.UNPROCESSABLE_ENTITY).build();
        }
    }

    @DeleteMapping("/delete/{id}")
    public ResponseEntity deleteTeacherById(@PathVariable("id") Long id) {
        try {
            teacherServiceImp.deleteTeacherById(id);
            return ResponseEntity.status(HttpStatus.NO_CONTENT).build();
        } catch (EmptyResultDataAccessException e) {
            return ResponseEntity.status(HttpStatus.UNPROCESSABLE_ENTITY).build();
        }
    }

}
