package com.school.project.service.userRegistration;

import com.school.project.dto.StudentRegistrationDTO;
import com.school.project.dto.UserDTO;
import com.school.project.model.registration.Role;
import com.school.project.model.registration.StudentReg;
import com.school.project.model.registration.User;
import com.school.project.repository.RoleRepository;
import com.school.project.repository.StudentRegRepository;
import com.school.project.repository.UserRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.UUID;

@Service
public class StudentRegistrationServiceImpl implements StudentRegistrationService{

    @Autowired
    private StudentRegRepository studentRegRepository;

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private RoleRepository roleRepository;

    @Autowired
    private PasswordEncoder passwordEncoder;

    @Override
    @Transactional
    public UserDTO registerStudent(StudentRegistrationDTO studentRegistrationDTO) {

        User newUser = new User();

        newUser.setEmail(studentRegistrationDTO.getEmail());
        newUser.setRole(roleRepository.findByRole(Role.ROLES.ADMIN.name()));

        newUser.setPassword(passwordEncoder.encode(studentRegistrationDTO.getPassword()));

        newUser = userRepository.save(newUser);

        StudentReg newStudentReg = new StudentReg();

        newStudentReg.setStudentAppUser(newUser);
        newStudentReg.setRollNumber(UUID.randomUUID().toString());

        studentRegRepository.save(newStudentReg);

        return new UserDTO(newUser.getUserId(), newUser.getEmail());

    }
}
